(* ::Package:: *)

BeginPackage["SMEFTsimFR`"]


(* ::Subsection::Closed:: *)
(*Initialization*)


alphaShifts::usage="replacements for {\[Alpha],mZ,GF} inputs"
MwShifts::usage="replacements for {mW,mZ,GF} inputs"


lhloop5::usage="FR from SM H loops, 5 legs"
lhloop5P::usage="FR from SM H loops, 5 legs, with dummy particles"
frBos::usage="Bosonic FR"
frBosP::usage="Bosonic FR with dummy particles"
fr4cpv::usage="CP violating FR from class 4"
fr1::usage="FR from class 1"
frFer::usage="FR from Fermionic L + class 7"
frFerP::usage="FR from Fermionic L + class 7 + dummy propagator particles"
frYuk::usage="FR from Yukawas + class 5"
frYukP::usage="FR from Yukawas + class 5 + dummy propagator particles"
fr6::usage="FR from class 6"
fr8a::usage="FR from class 8a"
fr8b::usage="FR from class 8b"
fr8c::usage="FR from class 8c"
fr8d::usage="FR from class 8d"
WC6::usage="List of all SMEFT parameters"
WC6cl1::usage="List of SMEFT parameters of class 1"
WC6cl23::usage="List of SMEFT parameters of class 2 and 3"
WC6cl4::usage="List of SMEFT parameters of class 4"
WC6cl5::usage="List of SMEFT parameters of class 5"
WC6cl6::usage="List of SMEFT parameters of class 6"
WC6cl7::usage="List of SMEFT parameters of class 7"
WC6cl8a::usage="List of SMEFT parameters of class 8a"
WC6cl8b::usage="List of SMEFT parameters of class 8b"
WC6cl8c::usage="List of SMEFT parameters of class 8c"
WC6cl8d::usage="List of SMEFT parameters of class 8d"
WCsimplify::usage="Function that collects Wilson coefficients and simplifies prefactors"
SMlimit::usage="takes SM limit"
relativeVariation::usage="Function that factors out SM expression"
simplifyLorentz::usage="Function that collects the SM Lorentz structure"
FermSelect::usage=" "
CKMto1::usage="function that sets CKM to identity"
massless::usage="function that sets to 0 all fermion masses except mt (and mb)"
CPonly::usage="function that removes CP odd WCs"
MergeFR::usage="function to merge FRs for the same vertex"
FRbyVertex::usage=" "
FRbyOperator::usage=" "


(* ::Text:: *)
(*import formatting from FeynRules*)


<<"FRFormat.m"


(* ::Subsection:: *)
(*FR*)


(* ::Subsubsection::Closed:: *)
(*bosonic *)


<<"FRloops.wl"


<<"FRbosonic.wl"


(* ::Subsubsection::Closed:: *)
(*fermionic*)


<<"FRfermionicGeneral.wl"


<<"FRfermionicU35.wl"


<<"FRfermionicMFV.wl"


<<"FRfermionicTop.wl"


<<"FRfermionicTopU3l.wl"


frFer[flavor_]:=Switch[flavor,
general,frFerGeneral,
U35,frFerU35,
MFV,frFerMFV,
top,frFerTop,
topU3l,frFerTopU3l]/.{tq->t,tqbar->tbar,bq->b,bqbar->bbar}//MapAt[ReplaceAll[#,G->gs]&,#,{All,2}]&;


frFerP[flavor_]:=Switch[flavor,
general,frFerPGeneral,
U35,frFerPU35,
MFV,frFerPMFV,
top,frFerPTop,
topU3l,frFerPTopU3l]/.{tq->t,tqbar->tbar,bq->b,bqbar->bbar}//MapAt[ReplaceAll[#,G->gs]&,#,{All,2}]&;


frYuk[flavor_]:=Switch[flavor,
general,frYukGeneral,
U35,frYukU35,
MFV,frYukMFV,
top,frYukTop,
topU3l,frYukTopU3l]/.{tq->t,tqbar->tbar,bq->b,bqbar->bbar};


frYukP[flavor_]:=Switch[flavor,
general,frYukPGeneral,
U35,frYukPU35,
MFV,frYukPMFV,
top,frYukPTop,
topU3l,frYukPTopU3l]/.{tq->t,tqbar->tbar,bq->b,bqbar->bbar};


fr6[flavor_]:=Switch[flavor,
general,fr6General,
U35,fr6U35,
MFV,fr6MFV,
top,fr6Top,
topU3l,fr6TopU3l]/.{tq->t,tqbar->tbar,bq->b,bqbar->bbar};


fr8a[flavor_]:=Switch[flavor,
general,fr8aGeneral,
U35,fr8aU35,
MFV,fr8aMFV,
top,fr8aTop,
topU3l,fr8aTopU3l]/.{tq->t,tqbar->tbar,bq->b,bqbar->bbar};


fr8b[flavor_]:=Switch[flavor,
general,fr8bGeneral,
U35,fr8bU35,
MFV,fr8bMFV,
top,fr8bTop,
topU3l,fr8bTopU3l]/.{tq->t,tqbar->tbar,bq->b,bqbar->bbar};


fr8c[flavor_]:=Switch[flavor,
general,fr8cGeneral,
U35,fr8cU35,
MFV,fr8cMFV,
top,fr8cTop,
topU3l,fr8cTopU3l]/.{tq->t,tqbar->tbar,bq->b,bqbar->bbar};


fr8d[flavor_]:=Switch[flavor,
general,fr8dGeneral,
U35,fr8dU35,
MFV,fr8dMFV,
top,fr8dTop,
topU3l,fr8dTopU3l]/.{tq->t,tqbar->tbar,bq->b,bqbar->bbar};


(* ::Subsubsection::Closed:: *)
(*inputs *)


alphaShifts={dMZ2->vevhat^2 (cHDD/2+2 cth sth cHWB)/LambdaSMEFT^2,
dGf->DGFdef,
dg1->sth^2/2/(1-2sth^2) (dGf+dMZ2+2 cth^2 cth/sth cHWB vevhat^2/LambdaSMEFT^2),
dgw->-cth^2/2/(1-2sth^2) (dGf+dMZ2+2 sth^2 sth/cth cHWB vevhat^2/LambdaSMEFT^2),
dMW->MWsm(dGf/2+dgw),dMH2->2 dkH-3/(2 lam) cH vevhat^2/LambdaSMEFT^2,
dkH->vevhat^2/LambdaSMEFT^2 (cHbox-cHDD/4)}

MwShifts={dMZ2->vevhat^2/LambdaSMEFT^2 (cHDD/2+2 cth sth cHWB),
dGf->DGFdef,
dg1->-(dGf+1/sth^2 dMZ2)/2,
dgw->-dGf/2,dMH2->2 dkH-3/(2 lam) cH vevhat^2/LambdaSMEFT^2,
dkH->vevhat^2/LambdaSMEFT^2 (cHbox-cHDD/4)}


dGfflavor[flav_]:=Switch[flav,
general, vevhat^2/LambdaSMEFT^2 (cHl3Re11 + cHl3Re22 - cllRe1221),
U35|MFV|topU3l,vevhat^2/LambdaSMEFT^2 (2 cHl3 - cll1),
top,vevhat^2/LambdaSMEFT^2 (cHl311 + cHl322 - cll1221/2)]


inp[inputs_,flav_]:=Switch[inputs,
mW,Join[MwShifts,{DGFdef->dGfflavor[flav]}],
alpha,Join[alphaShifts,{DGFdef->dGfflavor[flav]}],generic,{}];


(* ::Subsubsection::Closed:: *)
(*WC6*)


WC6 := Join[{dGf, dMZ2, dkH, dlam, dgw, dg1}, ToExpression[Names["SMEFTsimFR`c*"]],ToExpression[Names["SMEFTsimFR`Delta*"]]]//DeleteCases[#,c|cbar|cth|ceil|cos|cosh|cot|coth|counter|csc|csch] &


WC6cl1:={cG,cW,cGtil,cWtil};
WC6cl23:={cH,cHbox,cHDD};
WC6cl4:={cHG,cHW,cHB,cHWB,cHGtil,cHWtil,cHBtil,cHWBtil};
WC6cl5:=DeleteCases[Select[WC6,!FreeQ[frYukU35,#]||!FreeQ[frYukMFV,#]||!FreeQ[frYukTop,#]||!FreeQ[frYukTopU3l,#]||!FreeQ[frYukGeneral,#]&],{dGf, dMZ2, dkH, dlam, dgw, dg1}]
WC6cl7:=DeleteCases[Select[WC6,!FreeQ[frFerU35,#]||!FreeQ[frFerMFV,#]||!FreeQ[frFerTop,#]||!FreeQ[frFerTopU3l,#]||!FreeQ[frFerGeneral,#]&],{dGf, dMZ2, dkH, dlam, dgw, dg1}]
WC6cl6:=Select[WC6,!FreeQ[fr6U35,#]||!FreeQ[fr6MFV,#]||!FreeQ[fr6Top,#]||!FreeQ[fr6TopU3l,#]||!FreeQ[fr6General,#]&]
WC6cl8a:=Select[WC6,!FreeQ[fr8aU35,#]||!FreeQ[fr8aMFV,#]||!FreeQ[fr8aTop,#]||!FreeQ[fr8aTopU3l,#]||!FreeQ[fr8aGeneral,#]&]
WC6cl8b:=Select[WC6,!FreeQ[fr8bU35,#]||!FreeQ[fr8bMFV,#]||!FreeQ[fr8bTop,#]||!FreeQ[fr8bTopU3l,#]||!FreeQ[fr8bGeneral,#]&]
WC6cl8c:=Select[WC6,!FreeQ[fr8cU35,#]||!FreeQ[fr8cMFV,#]||!FreeQ[fr8cTop,#]||!FreeQ[fr8cTopU3l,#]||!FreeQ[fr8cGeneral,#]&]
WC6cl8d:=Select[WC6,!FreeQ[fr8dU35,#]||!FreeQ[fr8dMFV,#]||!FreeQ[fr8dTop,#]||!FreeQ[fr8dTopU3l,#]||!FreeQ[fr8dGeneral,#]&]


WCsimplify[x_] := Collect[x, Join[WC6, {gHgg1, gHgg2, gHgg3, gHgg4, gHgg5}], Simplify[Simplify[#,sth^2+cth^2==1]/.(-1+sth^2)->-cth^2]&]


SMlimit[x_,keepInputs_:False]:=Module[{tozero},
If[keepInputs,
tozero=WC6[[7;;]], tozero=WC6];
x//.Alternatives@@tozero->0
]


relativeVariation[x_] := Module[{sm},
  sm =SMlimit[x];
  If[sm === 0, x, sm (WCsimplify[x/sm])]
  ]


(* ::Subsubsection:: *)
(*simplification functions*)


FermSelect[FR_, fgenL_, sgenL_, tgenL_, fgenQ_, sgenQ_, tgenQ_] := 
 If[! fgenL && ! sgenL && ! tgenL && ! fgenQ && ! sgenQ && ! tgenQ, 
  FR, Select[FR, FreeQ[#[[1]],
     Alternatives@@Join[
       If[fgenL, {e, ebar, ve, vebar}, {}], 
       If[sgenL, {mu, mubar, vm, vmbar}, {}], 
       If[tgenL, {ta, tabar, vt, vtbar}, {}], 
       If[fgenQ, {u, ubar, d, dbar}, {}], 
       If[sgenQ, {s, sbar, c, cbar}, {}], 
       If[tgenQ, {t, tbar, b, bbar}, {}]
       ]] &]]


CKMto1[FR_,bool_]:=If[!bool,FR,ReplaceAll[FR,{Alternatives@@{CKM1x2,CKM1x3,CKM2x1,CKM2x3,CKM3x1,CKM3x2}->0,Alternatives@@{CKM1x1,CKM2x2,CKM3x3}->1}]]


massless[FR_,bool_]:=If[!bool,FR,ReplaceAll[FR,{Alternatives@@{yup,ydo,ys,yc,ye,ym,ytau}->0}]]


masslessB[FR_,bool_]:=If[!bool,FR,ReplaceAll[FR,{Alternatives@@{yb}->0}]]


CPonly[FR_,bool_]:=If[!bool,FR,ReplaceAll[FR,{Alternatives@@{cHWtil,cHBtil,cHWBtil,cHGtil,cWtil,cGtil}->0,Alternatives@@ToExpression[Names["SMEFTsimFR`c*Im*"]]->0}]]


RemoveSM[fr_,operators_List]:= Map[Coefficient[fr,#]&,operators].operators


MergeFR[FRlist_] := Module[{newList = {}, iiNew},
  (*check if the list of FR contains the same vertex multiple times *)
    If[! DuplicateFreeQ[FRlist[[;; , 1]]], 
   (*if yes, copy onto newList, element by element *)
   Do[If[MemberQ[newList[[;; , 1]], 
      FRlist[[iiOld, 1]]], (*for each element, check if vertex has already been copied *)
      (*if yes, find the position in newList, and replace the value adding the new term *)
     iiNew   = Position[newList[[;; , 1]], FRlist[[iiOld, 1]]][[1, 1]];
     newList = ReplacePart[newList, {iiNew, 2} -> (newList[[iiNew, 2]] + FRlist[[iiOld, 2]])],
     AppendTo[newList, FRlist[[iiOld]]] (* if the vertex is new, just copy it to the newList*)
     ],
    {iiOld, Length[FRlist]}]
   , newList = FRlist];(* if there are no duplicates, leave the FR list as is*)
  newList
  ]


simplifyLorentz[x_] := 
 Module[{gammaFix, lorentz, sm, smlorentz, x0, x1, x2},
  gammaFix = If[FreeQ[x, ProjM | ProjP], x, x /. {
       Ga[i1_, i2_, i3_] :> TensDot[Ga[i1], ProjM][i2, i3] + TensDot[Ga[i1], ProjP][i2, i3]
       }] // WCsimplify;  
  sm = gammaFix /. Alternatives @@ WC6 -> 0;
  If[sm === 0, x0 = gammaFix,
   smlorentz = Select[Variables[sm], ! FreeQ[#, Lorentz | FV | SP | Spin] &];
   If[smlorentz == {},
    x1 = gammaFix /. Alternatives @@ Select[Variables[gammaFix], ! FreeQ[#, Lorentz | FV | SP | Spin] &] -> 0,
    x1 = Select[gammaFix, 
      FreeQ[# /. Alternatives @@ smlorentz -> 1, 
        Lorentz | FV | SP | Spin] &]
    ];
   x2 = gammaFix - x1 // WCsimplify;
   x0 = If[smlorentz == {}, relativeVariation[x1], 
      Collect[x1, smlorentz, relativeVariation]] + x2;
   ];
  x0
  ]


(* ::Subsection:: *)
(*output functions*)


(* ::Subsubsection::Closed:: *)
(*style and useful functions*)


mainColor = Lighter[Blend[{Blue, Green}], .83];
boxColor = Lighter[Blend[{Blue, Green}], .7];
textSize = Medium;


fieldsTable := Grid[
{{"Higgs boson", H}, {"photon", A}, {"Z boson", Z}, {"W+ boson", W}, {"W- boson", Wbar}, {"gluon", G},
   {"up-type quarks", u, c, t}, {"up-type antiquarks", ubar, cbar, tbar}, 
   {"down-type quarks", d, s, b}, {"down-type antiquarks", dbar, sbar, bbar},
   {"charged leptons", e, mu, ta}, {"charged antileptons", ebar, mubar, tabar}, 
   {"neutrinos", ve, vm, vt}, {"antineutrinos", vebar, vmbar, vtbar},
   {"dummy particles (see Manual)", H1, Z1, W1, W1bar, t1, t1bar}},
  Dividers -> {{True, True, False, False, False, False, False, True},
    {True, False, False, False, False, False, True, False, False, False, False, False, False, False, True, True}}, 
  Alignment -> {{Right, Center}}, Spacings -> {1, 1.2}]


parameterDefinitions := Grid[{
   {ee, "the electromagnetic coupling constant"},
   {gs, "the strong coupling constant"},
   {sth, "the sine of the Weinberg angle"},
   {cth, "the cosine of the Weinberg angle"},
   {vevhat, 
    "the Higgs vacuum expectation value as determined from \!\(\*SubscriptBox[\(G\), \(F\)]\) = \!\(\*FractionBox[\(1\), \(\*SqrtBox[\(2\)]\\\ \*SuperscriptBox[\(vevhat\), \(2\)]\)]\) and \
normalized to vevhat = 246. GeV"},
   {LambdaSMEFT, "the SMEFT cutoff scale \[CapitalLambda]"},
   {"yup, yc, yt", "the Yukawa couplings of the up-type quarks"},
   {"ydo, ys, yb", "the Yukawa couplings of the down-type quarks"},
   {"ye, ym, ytau", "the Yukawa couplings of the charged leptons"},
   {"gHaa","Coefficient encoding the SM H\[Gamma]\[Gamma] loop function in \!\(\*SubscriptBox[\(m\), \(\(t\)\(\\\ \)\)]\)\[Rule] \[Infinity] limit, such that \[ScriptCapitalL] \[Superset] gHaa \!\(\*SubscriptBox[\(A\), \(\[Mu]\[Nu]\)]\)\!\(\*SuperscriptBox[\(A\), \(\[Mu]\[Nu]\)]\) \!\(\*FractionBox[\(H\), \(vevhat\)]\)"},
   {"gHza","Coefficient encoding the SM HZ\[Gamma] loop function in \!\(\*SubscriptBox[\(m\), \(t\)]\) \[Rule] \[Infinity] limit, such that \[ScriptCapitalL] \[Superset] gHza \!\(\*SubscriptBox[\(Z\), \(\[Mu]\[Nu]\)]\)\!\(\*SuperscriptBox[\(A\), \(\[Mu]\[Nu]\)]\) \!\(\*FractionBox[\(H\), \(vevhat\)]\)"},
   {"gHgg1","Coefficient encoding the dim5 term of the SM Hgg loop function in \!\(\*SubscriptBox[\(m\), \(t\)]\) \[Rule] \[Infinity] limit, such that \[ScriptCapitalL] \[Superset] gHgg1 \!\(\*SubscriptBox[SuperscriptBox[\(G\), \(a\)], \(\[Mu]\[Nu]\)]\)\!\(\*SuperscriptBox[\(G\), \(a\\\ \[Mu]\[Nu]\)]\) \!\(\*FractionBox[\(H\), \(vevhat\)]\)"},
   {"gHgg2","Coefficient encoding a dim7 term of the SM Hgg loop function in \!\(\*SubscriptBox[\(m\), \(t\)]\) \[Rule] \[Infinity] limit, such that \[ScriptCapitalL] \[Superset] gHgg2 \!\(\*SubscriptBox[\(D\), \(\[Sigma]\)]\)\!\(\*SubscriptBox[SuperscriptBox[\(G\), \(a\)], \(\(\[Mu]\[Nu]\)\(\\\ \)\)]\)\!\(\*SuperscriptBox[\(D\), \(\[Sigma]\)]\)\!\(\*SuperscriptBox[\(G\), \(a\\\ \[Mu]\[Nu]\)]\) \!\(\*FractionBox[\(H\), \(vevhat\)]\)"},
   {"gHgg3","Coefficient encoding a dim7 term of the SM Hgg loop function in \!\(\*SubscriptBox[\(m\), \(t\)]\) \[Rule] \[Infinity] limit, such that \[ScriptCapitalL] \[Superset] gHgg3 \!\(\*SubscriptBox[\(f\), \(abc\)]\)\!\(\*SubscriptBox[SuperscriptBox[\(G\), \(a\\\ \[Nu]\)], \(\[Mu]\)]\)\!\(\*SubscriptBox[SuperscriptBox[\(G\), \(b\\\ \[Sigma]\)], \(\[Nu]\)]\)\!\(\*SubscriptBox[SuperscriptBox[\(G\), \(c\\\ \[Mu]\)], \(\[Sigma]\)]\) \!\(\*FractionBox[\(H\), \(vevhat\)]\)"},
   {"gHgg4","Coefficient encoding a dim7 term of the SM Hgg loop function in \!\(\*SubscriptBox[\(m\), \(t\)]\) \[Rule] \[Infinity] limit, such that \[ScriptCapitalL] \[Superset] gHgg4 \!\(\*SuperscriptBox[\(D\), \(\[Mu]\)]\)\!\(\*SubscriptBox[SuperscriptBox[\(G\), \(a\)], \(\[Mu]\[Nu]\)]\) \!\(\*SubscriptBox[\(D\), \(\[Sigma]\)]\)\!\(\*SuperscriptBox[\(G\), \(a\\\ \[Sigma]\[Nu]\)]\) \!\(\*FractionBox[\(H\), \(vevhat\)]\)"},
   {"gHgg5","Coefficient encoding a dim7 term of the SM Hgg loop function in \!\(\*SubscriptBox[\(m\), \(t\)]\) \[Rule] \[Infinity] limit, such that \[ScriptCapitalL] \[Superset] gHgg5 \!\(\*SuperscriptBox[\(G\), \(a\\\ \[Mu]\[Nu]\)]\)\!\(\*SubscriptBox[\(D\), \(\[Nu]\)]\)\!\(\*SuperscriptBox[\(D\), \(\[Sigma]\)]\)\!\(\*SubscriptBox[SuperscriptBox[\(G\), \(a\)], \(\[Sigma]\[Mu]\)]\) \!\(\*FractionBox[\(H\), \(vevhat\)]\)"},
   {"propCorr","Flag parameter for interactions involving dummy fields, that carry propagator corrections"}
   }, Alignment -> Left, Dividers -> {{False, True}, None}]


(* ::Subsubsection:: *)
(*by operator*)


FRbyOperator:=Style[Manipulate[
  Module[{opSplit, ops={}, toRemove, frules, frulesF,frulesG, frulesNoManipulation = {},tab, tab1,tab2},
   opSplit = StringSplit[StringDelete[operator, WhitespaceCharacter], ","];
   ops = Join @@ Table[
       ToExpression /@ Select[ToString /@ WC6, StringMatchQ[#, opSplit[[i]]] &],
       {i, Length[opSplit]}];
   toRemove = Complement[WC6[[7;;]], ops]//DeleteCases[#, 
   Which[MemberQ[ops,cHWB|cHl3|cll1], dgw|dg1|dGf|dMZ2
   , MemberQ[ops,cHDD], dkH|dlam|dGf|dMZ2|dg1|dgw
   , MemberQ[ops,cHbox], dkH|dlam]
   ]&;
  
   frulesF = Select[Join[
      If[noSM, {}, lhloop5], 
      frBos, fr1, fr4cpv,
      frFer[Oflavor],
      frYuk[Oflavor],
      fr6[Oflavor],
      fr8a[Oflavor],
      fr8b[Oflavor],
      fr8c[Oflavor],
      fr8d[Oflavor]]//MergeFR, ! FreeQ[#, Alternatives @@ Join[ops, {dGf, dMZ2, dkH, dlam, dgw, dg1}]] &]/.Alternatives @@ toRemove -> 0;
  
   (* if the feynman rules contain G^5 or G^6, store them to a separate list, where they won't be manipulated. otherwise they never go through simplify *)
   (* if the feynman rules contain HG^4: if the SM expression is included, simplify that and store as not to be further manipulated. 
                                        if the SM expression is not included, treat it normally. *)
   If[MemberQ[frulesF[[;;, 1, ;;, 1]],{G,G,G,G,G}|{G,G,G,G,G,G}||{G,G,G,G,H}],
     frulesNoManipulation = If[!noSM, Select[frulesF, #[[1, ;;, 1]]==={G,G,G,G,G} || #[[1, ;;, 1]]==={G,G,G,G,G,G} || #[[1, ;;, 1]]==={G,G,G,G,H} &]
                                    , Select[frulesF, #[[1, ;;, 1]]==={G,G,G,G,G} || #[[1, ;;, 1]]==={G,G,G,G,G,G} &]
                              ];
     frulesG = If[!noSM, Select[frulesF, #[[1, ;;, 1]]=!={G,G,G,G,G} && #[[1, ;;, 1]]=!={G,G,G,G,G,G}&& #[[1, ;;, 1]]=!={G,G,G,G,H}&]
                       , Select[frulesF, #[[1, ;;, 1]]=!={G,G,G,G,G} && #[[1, ;;, 1]]=!={G,G,G,G,G,G}&]
                  ]
     ,
     frulesG = frulesF
   ];
   
   frules = FermSelect[frulesG, fgenL, sgenL, tgenL, fgenQ, sgenQ, tgenQ] //SortBy[#, Length[#[[1]]] &] &;
   
   tab = Select[frules //.inp[Oinputs, Oflavor], !FreeQ[#, Alternatives @@ ops] &]/.Alternatives @@ toRemove -> 0 // 
         CKMto1[#, OnoCKM] & // massless[#, Oml] & // masslessB[#, OmlB] & // DeleteCases[#, {_, 0}] &;
   If[noSM,
    tab1 = MapAt[RemoveSM[#, ops]&, tab, {All, 2}]// WCsimplify// DeleteCases[#, {_, 0}] &,
    tab1 = tab // WCsimplify// DeleteCases[#, {_, 0}] & // Select[#, !FreeQ[#, Alternatives @@ ops] &]&
    ];
   If[tab1 == {}, "Wilson coefficient not defined or all vertices evaluated to zero.",
    Switch[OoutForm,
     simplify, tab2 = tab1 // Simplify,
     byOp, tab2 = tab1 // WCsimplify,
     default, tab2 = MapAt[simplifyLorentz, tab1, {All, 2}]
     ]; 
    If[OinForm, TableForm[{#[[1]], InputForm[#[[2]]]} & /@ Join[tab2,frulesNoManipulation]], TableForm[Join[tab2,frulesNoManipulation]]]
    ]]
  
  ,
  Grid[{
    (* row 1 *)
    {Text[Style["Feynman Rules by Operator", Bold, Large]],
     
     Row[{Style["Choose Wilson coefficient(s):", Medium], 
       InputField[
        Dynamic[operator, Initialization -> (operator := "cHB")], String, 
        FieldHint -> "Specify a Wilson coefficient, in the notation of the UFO model.", 
        BaseStyle -> {textSize}, FieldSize -> Small, FrameMargins -> 5]
        }],
     
     Grid[{{Style["Flavor setup:", textSize],
        SetterBar[Dynamic[Oflavor], {U35, MFV, top, topU3l, general}, BaseStyle -> textSize]},
       {Style["EW input scheme:", textSize],
        SetterBar[Dynamic[Oinputs], 
        { mW -> Style["\!\(\*SubscriptBox[\(m\), \(W\)]\), \!\(\*SubscriptBox[\(m\), \(Z\)]\), \!\(\*SubscriptBox[\(G\), \(F\)]\)", textSize], 
          alpha -> Style["\!\(\*SubscriptBox[\(\[Alpha]\), \(em\)]\), \!\(\*SubscriptBox[\(m\), \(Z\)]\), \!\(\*SubscriptBox[\(G\), \(F\)]\)", textSize], 
          generic -> Style["generic", textSize]}]}},
      Alignment -> {Left}]},
    
    (* row 2 *)
    {Text[Style["Output options", textSize, Bold]], Text[Style["Simplifications", textSize, Bold]], SpanFromLeft},
    
    (* row 3 *)
    {Grid[{
       {Row[{Checkbox[Dynamic[noSM]], Style[" remove SM terms", textSize]}], SpanFromLeft},
       {Row[{Checkbox[Dynamic[OinForm]], Style[" display output in InputForm", textSize]}],  SpanFromLeft},
       {Style["Formatting:", textSize], 
        RadioButtonBar[Dynamic[OoutForm],
         {default -> Style["default", textSize], 
          byOp -> Style["by EFT param.", textSize], 
          simplify -> Style["simplify all", textSize]}],
        SpanFromLeft}}, Alignment -> {Left, Top}],
     
     Grid[{
       {Checkbox[Dynamic[OnoCKM]], Style["CKM = 1", textSize]},
       {Checkbox[Dynamic[Oml]],  Style["\!\(\*SubscriptBox[\(y\), \(light\)]\) = 0", textSize]},
       {Checkbox[Dynamic[OmlB]], Style["\!\(\*SubscriptBox[\(y\), \(b\)]\) = 0", textSize]}
       }, Alignment -> Left],
     
     Grid[{
       {Style["Omit fermions of generation:", textSize], SpanFromLeft, SpanFromLeft, SpanFromLeft},
       {Style["quarks:", textSize],
        Checkbox[Dynamic[fgenQ]], Style["1st", textSize],
        Checkbox[Dynamic[sgenQ]], Style["2nd", textSize],
        Checkbox[Dynamic[tgenQ]], Style["3rd", textSize]},
       {Style["leptons:", textSize],
        Checkbox[Dynamic[fgenL]], Style["1st", textSize],
        Checkbox[Dynamic[sgenL]], Style["2nd", textSize],
        Checkbox[Dynamic[tgenL]], Style["3rd", textSize]}}, 
      Alignment -> Left]
      }}, Frame -> True, Dividers -> {{True, True}, {True, True}}, Spacings -> {1.5, 1.5},Alignment -> Center,
   FrameStyle -> Darker[Gray, .4], Background -> {None, None, {{1, 2} -> boxColor, {1, 3} -> boxColor}}]
  ]
 , DefaultOptions -> {Panel -> {BaseStyle -> 18, Background -> mainColor}}]


(* ::Subsubsection::Closed:: *)
(*by vertex*)


FRbyVertex:=Style[Manipulate[
  Module[{vList, frules, frulesF, frulesNoManipulation = {}, tab, tab0, tab1, tab2 = {}, toRemove, WClist = {}},
   vList = Sort[ToExpression /@ StringSplit[vertex, ","]] ;
   toRemove = Join[If[noCl23, WC6cl23, {}], If[noCl4, WC6cl4, {}], (*needed for bosonic*) 
     If[noCl7, ToExpression /@ Names["SMEFTsimFR`cHl3*"], {}], 
     If[noCl8a, ToExpression /@ Names["SMEFTsimFR`cll*"], {}] (* needed for inputs *)
     ];
   frules = Select[Join[
      lhloop5, lhloop5P, frBosP, frFerP[Vflavor], frYukP[Vflavor],
      frBos /. Alternatives @@ toRemove -> 0,
      If[noCl1, {}, fr1],
      If[noCl4 || cp, {}, fr4cpv],
      If[noCl5, SMlimit[frYuk[Vflavor], True], frYuk[Vflavor]],
      If[noCl6, {}, fr6[Vflavor]],
      If[noCl7, SMlimit[frFer[Vflavor], True], frFer[Vflavor]],
      If[noCl8a, {}, fr8a[Vflavor]],
      If[noCl8b, {}, fr8b[Vflavor]],
      If[noCl8c, {}, fr8c[Vflavor]],
      If[noCl8d, {}, fr8d[Vflavor]]
      ], Sort[#[[1, ;; , 1]]] === vList&];
   
   tab0 = SortBy[frules, Length[#[[1]]] &];

   If[SMlim,
       If[MemberQ[{{G,G,G,G,G},{G,G,G,G,G,G},{G,G,G,G,H}}, vList],
          If[vList === {G,G,G,G,H},
           tab = SMlimit[tab0] // MergeFR //WCsimplify // DeleteCases[#, {_, 0}] &;
          ,tab = SMlimit[tab0] // MergeFR // DeleteCases[#, {_, 0}] &
          ];
        , tab = tab0 // SMlimit // WCsimplify // DeleteCases[#, {_, 0}] &
       ];
       tab2 = tab
       
      ,
      If[MemberQ[{{G,G,G,G,G},{G,G,G,G,G,G},{G,G,G,G,H}}, vList], 
          tab = tab0  // CPonly[#, cp] & // ReplaceAll[#, Alternatives @@ toRemove -> 0] &  // DeleteCases[#, {_, 0}] & // MergeFR;
          tab1 = tab0
         ,tab = tab0 //. inp[Vinputs, Vflavor] // 
              ReplaceAll[#, Alternatives @@ toRemove -> 0] & // 
              CPonly[#, cp] & // CKMto1[#, VnoCKM] & // 
              massless[#, Vml] & // masslessB[#, VmlB] & // MergeFR // WCsimplify // DeleteCases[#, {_, 0}] &
         ]
      ];
       
   If[tab == {}, "vertex does not exist or evaluated to zero",
      WClist = Select[WC6, ! FreeQ[tab, #] &];
      If[! SMlim, 
        If[!MemberQ[{{G,G,G,G,G},{G,G,G,G,G,G},{G,G,G,G,H}}, vList], 
            Switch[VoutForm,
              simplify, tab1 = tab // Simplify,
              byOp, tab1 = tab // WCsimplify,
              default, tab1 = MapAt[simplifyLorentz, tab, {All, 2}]]
            ];
         tab2 = Join[tab1, {{" ", Style["Wilson coefficients contributing: " <> ToString[WClist], textSize]}}];
         ];
      If[VinForm, TableForm[MapAt[InputForm, tab2, {1, 2}]], TableForm[tab2]]  
      ]       
   ]
  
  , Grid[{
    (* row 1 *)
    {Text[Style["Feynman Rules by Vertex", Bold, Large]],
     
     Row[{Style["Choose vertex:", Medium], 
       InputField[
        Dynamic[vertex, Initialization -> (vertex := "H,Z,Z")], String, 
        FieldHint ->  "List the particles in the vertex, using the notation above.", 
        BaseStyle -> {textSize}, FieldSize -> Small, FrameMargins -> 5]}],
     
     Grid[{{Style["Flavor setup:", textSize],
        SetterBar[Dynamic[Vflavor], {U35, MFV, top, topU3l, general}, 
         BaseStyle -> textSize]},
       {Style["EW input scheme:", textSize],
        SetterBar[Dynamic[Vinputs], 
         {mW -> Style["\!\(\*SubscriptBox[\(m\), \(W\)]\), \!\(\*SubscriptBox[\(m\), \(Z\)]\), \!\(\*SubscriptBox[\(G\), \(F\)]\)", textSize], 
          alpha -> Style["\!\(\*SubscriptBox[\(\[Alpha]\), \(em\)]\), \!\(\*SubscriptBox[\(m\), \(Z\)]\), \!\(\*SubscriptBox[\(G\), \(F\)]\)", textSize], 
          generic -> Style["generic", textSize]}
          ]}},
      Alignment -> {Left}]},
    
    (* row 2 *)
    {Text[Style["Output options", textSize, Bold]], Text[Style["Simplifications", textSize, Bold]], SpanFromLeft},
    
    (* row 3 *)
    {Grid[{
       {Row[{Checkbox[Dynamic[SMlim]], Style[" SM value only", textSize]}], SpanFromLeft},
       {Row[{Checkbox[Dynamic[VinForm]], Style[" display output in InputForm", textSize]}], SpanFromLeft},
       {Style["Formatting:", textSize], RadioButtonBar[Dynamic[VoutForm],
         {default -> Style["default", textSize], 
          byOp -> Style["by EFT param.", textSize], 
          simplify -> Style["simplify all", textSize]}],
        SpanFromLeft}}
      , Alignment -> {Left, Top}],
     
     Grid[{
       {Checkbox[Dynamic[cp]], Style["CP conserving only", textSize]},
       {Checkbox[Dynamic[VnoCKM]], Style["CKM = 1", textSize]},
       {Checkbox[Dynamic[Vml]],  Style["\!\(\*SubscriptBox[\(y\), \(light\)]\) = 0", textSize]},
       {Checkbox[Dynamic[VmlB]], Style["\!\(\*SubscriptBox[\(y\), \(b\)]\) = 0", textSize]}
       }, Alignment -> Left],
     
     Grid[{
       {Style["Omit operators of class:", textSize], SpanFromLeft, SpanFromLeft, SpanFromLeft},
       {Style["(bosonic)", Small], 
        Checkbox[Dynamic[noCl1]], Style["1", textSize], Checkbox[Dynamic[noCl23]], Style["2, 3", textSize], Checkbox[Dynamic[noCl4]], Style["4", textSize]},
       {Style["(2 fermions)", Small], 
        Checkbox[Dynamic[noCl5]], Style["5", textSize], Checkbox[Dynamic[noCl6]], Style["6", textSize], Checkbox[Dynamic[noCl7]], Style["7", textSize]},
       {Style["(4 fermions)", Small], 
       Checkbox[Dynamic[noCl8a]],  Style["8a", textSize], Checkbox[Dynamic[noCl8b]], Style["8b", textSize],
        Checkbox[Dynamic[noCl8c]], Style["8c", textSize], Checkbox[Dynamic[noCl8d]], Style["8d", textSize]}}, 
      Alignment -> Left]
      }}, Frame -> True, Dividers -> {{True, True}, {True, True}}, Spacings -> {1.5, 1.5}, Alignment -> Center,
   FrameStyle -> Darker[Gray, .4], Background -> {None, None, {{1, 2} -> boxColor, {1, 3} -> boxColor}}]
  ]
 , DefaultOptions -> {Panel -> {BaseStyle -> 18, Background -> mainColor}}]


EndPackage[]
